#!/usr/bin/perl -w

use warnings;
use strict;

use CGI;
use DBI;
use UNIVERSAL qw(isa);

my $dbh = DBI->connect('dbi:SQLite:dbname=db/tasks.db','','')
    or die "Can't open database tasks: " . $DBI::errstr . "\n";

my $cgi = CGI->new();
my @names = $cgi->param;

my $do_sync = $cgi->param('sync') ? 1 : 0;
my @has_new = grep { defined($_) && /^n(\d+)_s/ && $1 >= 2 } @names;
if ($cgi->param('add_new') || @has_new) {
    $do_sync = 1;

    my ($default_priority) = $dbh->selectrow_array(qq(SELECT COALESCE(MAX(priority),0)+1 FROM tasks));
    foreach my $name (@names) {
        next unless defined($name) && $name =~ /^n(\d+)_s$/;
        my $id = $1;

        my ($pri, $done, $sum, $desc) = do_fields($cgi, "n${id}", { priority => $default_priority });
        if ((defined($sum) && $sum =~ /\S/) || (defined($desc) && $desc =~ /\S/)) {
            $dbh->do( 'INSERT INTO tasks (priority, summary, description, complete) VALUES (?,?,?,?)',
                undef, $pri, $sum, $desc, $done ) or die $dbh->errstr;
        }
    }
}

if ($do_sync) {
    foreach my $name (@names) {
        next unless defined($name) && $name =~ /^t(\d+)_s$/;
        my $id = $1;
        my $db_task = $dbh->selectrow_hashref(<<'SQL', undef, $id);
            SELECT priority, summary, description
            FROM tasks
            WHERE taskid = ? AND visible = 1 AND (complete IS NULL OR NOT complete)
SQL
        die $dbh->errstr if $dbh->errstr;
        next unless defined($db_task);

        my ($pri, $done, $sum, $desc) = do_fields($cgi, "t${id}", $db_task);

        if ( $done ||
            (defined($pri) && ($pri != $db_task->{'priority'})) ||
            $sum ne $db_task->{'summary'} ||
            $desc ne $db_task->{'description'}
        ) {
            $dbh->do( 'UPDATE tasks SET priority=?, complete=?, summary=?, description=? WHERE taskid = ?',
                undef, $pri, $done, $sum, $desc, $id) or die $dbh->errstr;
        }
    }
}

my $taskrows = '';
my $tasks = $dbh->selectall_arrayref(<<'SQL',{ Slice => {} });
    SELECT taskid, summary, priority, description, complete
    FROM tasks
    WHERE visible = 1 AND (complete IS NULL OR NOT complete)
    ORDER BY priority, taskid
SQL

my $max_task = 0;
if ($tasks) {
    $max_task = scalar @{$tasks};
    my $cur_task = 0;
    foreach my $task (@{$tasks}) {
        $cur_task++;
        my $id = $task->{'taskid'};
        my $done = $task->{'complete'} ? 'checked="checked"' : '';
        my $summary = CGI::escapeHTML($task->{'summary'});
        my $description = CGI::escapeHTML($task->{'description'});

        my $selectrows = '';
        foreach my $i (1..$max_task) {
            $selectrows .= qq{<option value="$i"};
            if ($i == $cur_task) {
                $selectrows .= qq{ selected="selected"};
            }
            $selectrows .= qq{>$i</option>\n};
        }

        $taskrows .= <<"TASK";
<tr class="task_row">
    <td>
        <select id="t${id}_p" name="t${id}_p" onchange="adjust_priority(this);" class="sel_pri">$selectrows</select>
    </td>
    <td class="complete"><input type="checkbox" id="t${id}_c" name="t${id}_c" value="1" $done></td>
    <td class="summary"><input type="text" id="t${id}_s" name="t${id}_s" value="$summary" size="30"></td>
    <td class="description"><textarea id="t${id}_d" name="t${id}_d" rows="3" cols="50">$description</textarea></td>
</tr>
TASK
    }
}

my $html = join('',<DATA>);

$html =~ s/<!-- TASKLIST -->/$taskrows/;

print $cgi->header('text/html');
print $html;
exit;

sub do_fields {
    my ($cgi, $prefix, $defaults) = @_;
    my ($pri, $done, $sum, $desc);

    unless (defined($defaults) && isa($defaults, 'HASH')) {
        $defaults = {};
    }

    my $in_pri = $cgi->param("${prefix}_p");
    if (defined($in_pri) && $in_pri =~ /^(\d+)$/) {
        $pri = $1;
    }
    elsif (exists($defaults->{'priority'})) {
        $pri = $defaults->{'priority'};
    }

    my $in_done = $cgi->param("${prefix}_c");
    if ($in_done) {
        $done = 1;
    }

    my $in_sum = $cgi->param("${prefix}_s");
    if (defined($in_sum) && $in_sum =~ /\A(.*)\Z\n*/ms) {
        $sum = $1;
    }
    elsif (exists($defaults->{'summary'})) {
        $sum = $defaults->{'summary'};
    }
    else {
        $sum = '';
    }

    my $in_desc = $cgi->param("${prefix}_d");
    if (defined($in_desc) && $in_desc =~ /\A(.*)\Z\n*/ms) {
        $desc = $1;
    }
    elsif (exists($defaults->{'description'})) {
        $desc = $defaults->{'description'};
    }
    else {
        $desc = '';
    }

    return ($pri, $done, $sum, $desc);
}


__DATA__
<!DOCTYPE html>
<html>
    <head>
        <meta charset="iso-8859-1">
        <title>My Prioritized To-Do List</title>
        <style>
body { font-family: "Helvetica",sans-serif }
td.add, td.complete { text-align: center; vertical-align: middle; }
td.summary { vertical-align: top; }
table { border-collapse: collapse; }
th, td { border: 1px solid black; padding: 1px }
#tasktable th { background-color: #CCC; }
tr.template_row { display: none; }
        </style>
        <script src="jquery-1.7.2.min.js"></script>
        <script src="json2.min.js"></script>
        <script>
function adjust_priority(e) {
    var new_value = e.value;
    var old_value = 1;
    var all_selects = document.getElementsByTagName('select');
    var num_selects = all_selects.length;
    while (1) {
        for (var i = 0; i < num_selects; i++) {
            if (all_selects[i].value == old_value) {
                old_value++;
                continue;
            }
        }
        break;
    }
    if (old_value > num_selects) {
        return;
    }
    var from = old_value;
    var to = new_value;
    var dir = 'd'; // decreasing
    if (old_value > new_value) {
        from = new_value;
        to = old_value;
        dir = 'i'; /* increasing */
    }
    for (select_index in all_selects) {
	var select = all_selects[select_index];
        if (select.id != e.id) {
            if (select.value >= from && select.value <= to) {
                if (dir == 'i') { // increasing values
                    select.value++;
                }
                else {
                    select.value--;
                }
            }
        }
    }
    return true;
}

function add_new_task_row( taskid, new_complete, new_summary, new_description ) {
    // This adds a new priority to each existing one
    var all_selects = document.getElementsByTagName('select');
    var num_selects = all_selects.length;
    var new_pri = num_selects;
    $("select.sel_pri").append('<option value="' + new_pri + '">' + new_pri + '</option>');

    // This duplicates the template row and then modifies it to be our new row
    var new_task_row = $("tr.template_row :last").closest('tr').clone();
    var pre = 't' + taskid;
    new_task_row.attr('class','task_row');
    new_task_row.attr('id',pre+'_row');
    var new_select = new_task_row.find('select');
    new_select.empty();
    for (var i = 1; i <= new_pri; i++) {
        new_select.append('<option value="' + i + '">' + i + '</option');
    }
    new_select.attr('id',pre+'_p').attr('name',pre+'_p').val(new_pri);
    new_task_row.find('.complete input').attr('id',pre+'_c').attr('name',pre+'_c').prop('checked',new_complete);
    new_task_row.find('.summary input').attr('id',pre+'_s').attr('name',pre+'_s').val(new_summary);
    new_task_row.find('.description textarea').attr('id',pre+'_d').attr('name',pre+'_d').val(new_description);
    new_task_row.change(function(){
        return adjust_priority(this);
    });
    $("tbody").append(new_task_row);
    $('#'+pre+'_row').show(500);
    return true;
}

$(document).ready( function() {
    $("#add_new").click( function(event) {
	event.preventDefault();
        // This stores the new task values and resets the "add" fields
        var new_complete = $("#n1_c").is(':checked');
        if (new_complete) {
            $("#n1_c").toggle();
        }
        var new_summary = $("#n1_s").val();
        $("#n1_s").val('');
        var new_description = $("#n1_d").val();
        $("#n1_d").val('');

        // This uses JSON to add the row and get the taskid back
        $.getJSON('new_task.cgi', {
            summary: new_summary,
            description: new_description,
        },
        function(json){
            if (json.status && json.status == 'OK') {
                add_new_task_row( json.taskid, new_complete, new_summary, new_description );
            }
            else {
                alert("Error! Your new task may not have been saved.");
            }
        }).error(function() { alert("Error! Asynchronous request could not be sent"); });

        return;
    });
});
        </script>
    </head>
    <body>
        <h1 id="title">My Prioritized To-Do List</h1>
        <form method="post" action="index.cgi">
            <div id="topsync" class="sync"><input type="submit" name="sync" value="&#10227;"></div>
            <table id="tasktable">
                <thead>
                    <tr>
                        <th>Pri</th>
                        <th>&#10003;</th>
                        <th>Summary</th>
                        <th>Description</th>
                    </tr>
                </thead>
                <tfoot>
                    <tr>
                        <td style="background-color: #CCC;" colspan="4">Add a task:</td>
                    </tr>
                    <tr>
                        <td class="add"><input type="submit" id="add_new" name="add_new" value="+"></td>
                        <td class="complete"><input type="checkbox" id="n1_c" name="n1_c" value="done"></td>
                        <td class="summary"><input type="text" id="n1_s" name="n1_s" value="" size="30"></td>
                        <td class="description"><textarea id="n1_d" name="n1_d" rows="3" cols="50"></textarea></td>
                    </tr>
                </tfoot>
                <tbody>
                    <tr class="template_row">
                        <td><select onchange="adjust_priority(this);" class="sel_pri"></select></td>
                        <td class="complete"><input type="checkbox" value="1"></td>
                        <td class="summary"><input type="text" value="" size="30"></td>
                        <td class="description"><textarea rows="3" cols="50"></textarea></td>
                    </tr>
<!-- TASKLIST -->
                </tbody>
            </table>
            <div id="botsync" class="sync"><input type="submit" name="sync" value="&#10227;"></div>
        </form>
    </body>
</html>
