#!/usr/bin/perl -w

use warnings;
use strict;

use CGI;
use DBI;
use JSON;
use UNIVERSAL qw(isa);

my $dbh = DBI->connect('dbi:SQLite:dbname=db/tasks.db','','', { RaiseError => 1, PrintError => 1 })
    or die "Can't open database tasks: " . $DBI::errstr . "\n";

my $out = {
    status => 'ERROR',
    taskid => 0,
};

my $cgi = CGI->new();
my $sum = $cgi->param('summary');
$sum = defined($sum) ? $sum : '';
my $desc = $cgi->param('description');
$desc = defined($desc) ? $desc : '';
if ( $sum =~ /\S/ || $desc =~ /\S/ ) {
    my @binds;
    my $sql = 'SELECT 1 FROM tasks WHERE visible = 1 AND (complete IS NULL OR NOT complete)';
    if ($sum =~ /\S/) {
        $sql .= ' AND summary = ?';
        push @binds, $sum;
    }
    else {
        $sql .= ' AND (summary IS NULL OR summary = ?)';
        push @binds, '';
    }
    if ($desc =~ /\S/) {
        $sql .= ' AND description = ?';
        push @binds, $desc;
    }
    else {
        $sql .= ' AND (description IS NULL OR description = ?)';
        push @binds, '';
    }

    my ($db_task) = $dbh->selectrow_array($sql, undef, @binds);
    die $dbh->errstr if defined($dbh->errstr);
    if (!defined($db_task)) {
        my ($pri) = $dbh->selectrow_array(qq(SELECT COALESCE(MAX(priority),0)+1 FROM tasks));
        $dbh->do( 'INSERT INTO tasks (priority, summary, description, complete, visible) VALUES (?,?,?,0,1)',
            undef, $pri, $sum, $desc ) or die $dbh->errstr;
        my $new_taskid = $dbh->last_insert_id("","","","");
        if ($new_taskid > 0) {
            $out = {
                status => 'OK',
                taskid => $new_taskid,
            };
        }
    }
}

my $out_json = encode_json($out);
print <<JSON;
Content-type: application/json

$out_json
JSON

exit(0);
